==> hobson-0.8.10.tgz: package/lib/utils/helpers.js <==
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('mongoose'),
    Types = _require.Types;

var HTTPStatus = require('http-status');
var errors = require('../errors');

/**
 * Check an parameter is a string or throw an error.
 */
module.exports.expect = function expect() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref.name,
      value = _ref.value,
      _ref$type = _ref.type,
      type = _ref$type === undefined ? 'string' : _ref$type,
      _ref$optional = _ref.optional,
      optional = _ref$optional === undefined ? false : _ref$optional;

  if (optional && typeof value === 'undefined') {
    return;
  }
  if (typeof name !== 'string') {
    throw new errors.Response({ message: 'Expected value "name" to be of type string but got ' + (typeof name === 'undefined' ? 'undefined' : _typeof(name)) + '.' });
  }
  if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) !== type) {
    throw new errors.Response({ message: 'Expected value "' + name + '" to be of type ' + type + ' but got ' + (typeof value === 'undefined' ? 'undefined' : _typeof(value)) + '.' });
  }
};

/**
 * Check if the id of an item is a valid.
 */
module.exports.checkObjectId = function checkObjectId(id) {
  if (!id || !Types.ObjectId.isValid(id)) {
    throw new errors.Response({ message: 'Expected value "id" to be a valid ObjectId but got ' + id + '.' });
  }
};

/**
 * Format response.
 */
module.exports.formatResponse = function formatResponse() {
  var response = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var debug = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (response instanceof Error) {
    var _status = response.status,
        _code = response.code,
        _data = response.data,
        message = response.message,
        stack = response.stack;

    var error = {
      status: _status || HTTPStatus.INTERNAL_SERVER_ERROR,
      code: _code || 'error',
      message: message || 'Error has occurred with status ' + (_status || 'unknown') + '.'
    };
    if (debug && stack) {
      error.stack = stack;
    }
    if (_data) {
      error.data = _data;
    }
    return error;
  }
  var status = response.status,
      code = response.code,
      data = response.data;

  return {
    status: status || data ? HTTPStatus.OK : HTTPStatus.NO_CONTENT,
    code: code || 'success',
    data: data
  };
};

/**
 * Format middleware to match express infrastructure.
 */
module.exports.middlify = function middlify(middleware, resources) {
  var _this = this;

  var finish = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var formatResponse = module.exports.formatResponse;

  var exec = function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      var _args = arguments;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt('return', middleware.apply(undefined, _args));

            case 1:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function exec() {
      return _ref2.apply(this, arguments);
    };
  }();
  return function (req, res, next) {
    var options = _extends({
      req: req,
      res: res,
      next: next,
      body: req.body || {},
      params: req.params || {},
      query: req.query || {},
      user: req.user,
      auth: req.auth
    }, resources);
    exec(options).then(function (data) {
      if (finish) {
        var response = formatResponse({ data: data });
        res.status(response.status || HTTPStatus.INTERNAL_SERVER_ERROR).json(response);
      } else {
        next();
      }
    }).catch(next);
  };
};

/**
 * Format hooks and execute work.
 */
module.exports.hookify = function hookify(key, handler, befores, afters) {
  var _this2 = this;

  return function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(options) {
      var tasksBefore, data, tasksAfter;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              tasksBefore = befores.map(function (hook) {
                return hook(options);
              });
              _context2.next = 3;
              return Promise.all(tasksBefore);

            case 3:
              data = void 0;
              _context2.prev = 4;
              _context2.next = 7;
              return handler(options);

            case 7:
              data = _context2.sent;
              _context2.next = 17;
              break;

            case 10:
              _context2.prev = 10;
              _context2.t0 = _context2['catch'](4);

              if (!(_context2.t0 && _context2.t0.name === 'ValidationError')) {
                _context2.next = 14;
                break;
              }

              throw new errors.Response({
                message: _context2.t0._message || 'Request validation failed.',
                status: HTTPStatus.BAD_REQUEST,
                data: _context2.t0.errors
              });

            case 14:
              if (!(_context2.t0 && _context2.t0.name === 'MongoError')) {
                _context2.next = 16;
                break;
              }

              throw new errors.Response({
                message: _context2.t0.message || 'Error occurred when working with database.',
                status: HTTPStatus.BAD_REQUEST,
                data: _context2.t0.errors
              });

            case 16:
              throw _context2.t0 || new errors.Response({
                message: 'Error occurred on the server.',
                status: HTTPStatus.INTERNAL_SERVER_ERROR
              });

            case 17:
              Object.assign(options, { data: data });
              tasksAfter = afters.map(function (hook) {
                return hook(options);
              });
              _context2.next = 21;
              return Promise.all(tasksAfter);

            case 21:
              return _context2.abrupt('return', data);

            case 22:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2, [[4, 10]]);
    }));

    return function (_x5) {
      return _ref3.apply(this, arguments);
    };
  }();
};

/**
 * Check permissions.
 */
module.exports.permissionify = function permissionify(key, permissions, defaultOpen) {
  var _this3 = this;

  return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var checks, status;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            checks = permissions.map(function (check) {
              return check.apply(undefined, _toConsumableArray(args));
            });
            _context3.next = 3;
            return Promise.all(checks);

          case 3:
            status = _context3.sent;

            if (!(!defaultOpen && !status.length || status.length !== status.filter(function (outcome) {
              return Boolean(outcome);
            }).length)) {
              _context3.next = 6;
              break;
            }

            throw new errors.Response({
              message: 'Permission denied to route.',
              status: HTTPStatus.UNAUTHORIZED
            });

          case 6:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, _this3);
  }));
};

/**
 * Order express routes correctly so they execute correctly
 */
module.exports.orderRoutes = function orderRoutes(a, b) {
  var aSegments = a[1].path.split('/').filter(function (segment) {
    return segment.length;
  });
  var bSegments = b[1].path.split('/').filter(function (segment) {
    return segment.length;
  });
  if (aSegments.length && aSegments.length === bSegments.length) {
    var index = 0;
    while (index < aSegments.length) {
      if (aSegments[index].charAt(0) === ':' && bSegments[index].charAt(0) !== ':') {
        return 1;
      }
      if (bSegments[index].charAt(0) === ':' && aSegments[index].charAt(0) !== ':') {
        return -1;
      }
      index += 1;
    }
  }
  return bSegments.length - aSegments.length;
};

/**
 * Email validation regex.
 */
module.exports.emailRegex = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;

==> hobson-0.9.1.tgz: package/lib/utils/helpers.js <==
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _require = require('mongoose'),
    Types = _require.Types;

var HTTPStatus = require('http-status');
var errors = require('../errors');

/**
 * Check an parameter is a string or throw an error.
 */
module.exports.expect = function expect() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref.name,
      value = _ref.value,
      _ref$type = _ref.type,
      type = _ref$type === undefined ? 'string' : _ref$type,
      _ref$optional = _ref.optional,
      optional = _ref$optional === undefined ? false : _ref$optional;

  if (optional && typeof value === 'undefined') {
    return;
  }
  if (typeof name !== 'string') {
    throw new errors.BreakingResponse({ message: 'Expected value "name" to be of type string but got ' + (typeof name === 'undefined' ? 'undefined' : _typeof(name)) + '.' });
  }
  if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) !== type) {
    throw new errors.BreakingResponse({ message: 'Expected value "' + name + '" to be of type ' + type + ' but got ' + (typeof value === 'undefined' ? 'undefined' : _typeof(value)) + '.' });
  }
};

/**
 * Check if the id of an item is a valid.
 */
module.exports.checkObjectId = function checkObjectId(id) {
  if (!id || !Types.ObjectId.isValid(id)) {
    throw new errors.BreakingResponse({ message: 'Expected value "id" to be a valid ObjectId but got ' + id + '.' });
  }
};

/**
 * Send a depreciated error.
 */
module.exports.depreciated = function depreciated() {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      name = _ref2.name,
      value = _ref2.value;

  if (typeof value !== 'undefined') {
    throw new errors.BreakingResponse({ message: 'The value "' + name + '" has been depreciated.' });
  }
};

/**
 * Format response.
 */
module.exports.formatResponse = function formatResponse() {
  var meta = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var debug = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (meta instanceof Error) {
    var _status = meta.status,
        _code = meta.code,
        _data = meta.data,
        message = meta.message,
        stack = meta.stack;

    var error = {
      status: _status || HTTPStatus.INTERNAL_SERVER_ERROR,
      code: _code || 'error',
      message: message || 'Error has occurred with status ' + (_status || 'unknown') + '.'
    };
    if (debug && stack) {
      error.stack = stack;
    }
    if (_data) {
      error.data = _data;
    }
    return error;
  }
  var status = meta.status,
      code = meta.code,
      data = meta.data;

  return {
    status: status || data ? HTTPStatus.OK : HTTPStatus.NO_CONTENT,
    code: code || 'success',
    data: data
  };
};

/**
 * Format middleware to match express infrastructure.
 */
module.exports.middlify = function middlify(middleware, resources) {
  var _this = this;

  var finish = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var formatResponse = module.exports.formatResponse;

  var exec = function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      var _args = arguments;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt('return', middleware.apply(undefined, _args));

            case 1:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function exec() {
      return _ref3.apply(this, arguments);
    };
  }();
  return function (req, res, next) {
    var options = _extends({
      req: req,
      res: res,
      next: next,
      body: req.body || {},
      params: req.params || {},
      query: req.query || {},
      user: req.user,
      auth: req.auth
    }, resources);
    exec(options).then(function (data) {
      if (finish) {
        var response = formatResponse({ data: data });
        res.status(response.status || HTTPStatus.INTERNAL_SERVER_ERROR).json(response);
      } else {
        next();
      }
    }).catch(next);
  };
};

/**
 * Format hooks and execute work.
 */
module.exports.hookify = function hookify(key, handler, befores, afters) {
  var _this2 = this;

  return function () {
    var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(options) {
      var tasksBefore, data, tasksAfter;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              tasksBefore = befores.map(function (hook) {
                return hook(options);
              });
              _context2.next = 3;
              return Promise.all(tasksBefore);

            case 3:
              data = void 0;
              _context2.prev = 4;
              _context2.next = 7;
              return handler(options);

            case 7:
              data = _context2.sent;
              _context2.next = 17;
              break;

            case 10:
              _context2.prev = 10;
              _context2.t0 = _context2['catch'](4);

              if (!(_context2.t0 && _context2.t0.name === 'ValidationError')) {
                _context2.next = 14;
                break;
              }

              throw new errors.BadResponse({
                message: _context2.t0._message || 'Request validation failed.',
                data: _context2.t0.errors
              });

            case 14:
              if (!(_context2.t0 && _context2.t0.name === 'MongoError')) {
                _context2.next = 16;
                break;
              }

              throw new errors.BadResponse({
                message: _context2.t0.message || 'Error occurred when working with database.',
                data: _context2.t0.errors
              });

            case 16:
              throw _context2.t0 || new errors.BreakingResponse({ message: 'Error occurred on the server.' });

            case 17:
              Object.assign(options, { data: data });
              tasksAfter = afters.map(function (hook) {
                return hook(options);
              });
              _context2.next = 21;
              return Promise.all(tasksAfter);

            case 21:
              return _context2.abrupt('return', data);

            case 22:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this2, [[4, 10]]);
    }));

    return function (_x6) {
      return _ref4.apply(this, arguments);
    };
  }();
};

/**
 * Check permissions.
 */
module.exports.permissionify = function permissionify(key, permissions, defaultOpen) {
  var _this3 = this;

  return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var checks, status;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            checks = permissions.map(function (check) {
              return check.apply(undefined, _toConsumableArray(args));
            });
            _context3.next = 3;
            return Promise.all(checks);

          case 3:
            status = _context3.sent;

            if (!(!defaultOpen && !status.length || status.length !== status.filter(function (outcome) {
              return Boolean(outcome);
            }).length)) {
              _context3.next = 6;
              break;
            }

            throw new errors.UnauthResponse({ message: 'Permission denied to route.' });

          case 6:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, _this3);
  }));
};

/**
 * Order express routes correctly so they execute correctly
 */
module.exports.orderRoutes = function orderRoutes(a, b) {
  var aSegments = a[1].path.split('/').filter(function (segment) {
    return segment.length;
  });
  var bSegments = b[1].path.split('/').filter(function (segment) {
    return segment.length;
  });
  if (aSegments.length && aSegments.length === bSegments.length) {
    var index = 0;
    while (index < aSegments.length) {
      if (aSegments[index].charAt(0) === ':' && bSegments[index].charAt(0) !== ':') {
        return 1;
      }
      if (bSegments[index].charAt(0) === ':' && aSegments[index].charAt(0) !== ':') {
        return -1;
      }
      index += 1;
    }
  }
  return bSegments.length - aSegments.length;
};

/**
 * Email validation regex.
 */
module.exports.emailRegex = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
